
#import "Friio.h"

@implementation Friio

// 設定値定義
// -----------------------------------------------------------
static const int PIPE_REF = 1;
static const int BUFFER_SIZE = 327680;
static const int RESPONSE_SIZE = 16;

static int RESPONSE_NUM = 0;
static int THREAD_FLAG = 0;
static int TIMER_FLAG = 0;

// レスポンス配列を生成
static USBResponse res[16];


// オープンメソッド
// -----------------------------------------------------------
- (int)open
{
	int result = 0;
	
	// デバイスを検索
	result = [self _findDevice];
	if(!result) {
		return 0;
	}
	
	// インターフェースを検索
	result = [self _findInterface];
	if(!result) {
		return 0;
	}
	
	// インターフェースをセットアップ
	result = [self _setupInterface];
	if(!result) {
		return 0;
	}
	
	return 1;
}


// デバイスリファレンス取得メソッド
// -----------------------------------------------------------
- (int)_findDevice
{
	SInt32					idVendor = 0x7A69;
    SInt32					idProduct = 0x0001;
	kern_return_t			error;
	CFNumberRef				numberRef;
	CFMutableDictionaryRef 	matchingDictionary = 0;
    io_iterator_t			iterator = 0;
	
	// マスターポートを生成
	error = IOMasterPort(MACH_PORT_NULL, &masterPort);
	if(error) {
		printf("_findDevice: Could not open master port.\n");
		return 0;
	}
	
	// 辞書を生成
	matchingDictionary = IOServiceMatching(kIOUSBDeviceClassName);
	// ベンダーIDを辞書に登録
	numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &idVendor);
	CFDictionaryAddValue(matchingDictionary, CFSTR(kUSBVendorID), numberRef);
    CFRelease(numberRef);
	numberRef = 0;
	// プロダクトIDを辞書に登録
    numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &idProduct);
	CFDictionaryAddValue(matchingDictionary, CFSTR(kUSBProductID), numberRef);
    CFRelease(numberRef);
    numberRef = 0;
	
	// Friioを探す
	error = IOServiceGetMatchingServices(masterPort, matchingDictionary, &iterator);
	matchingDictionary = 0;
	if(error) {
		printf("_findDevice: Unable to find the device of the friio.\n");
		return 0;
	}
	
	// USBデバイスのリファレンスを取得
	usbDeviceRef = IOIteratorNext(iterator);
	
	// 後処理
	IOObjectRelease(iterator);
	iterator = 0;
	
	return 1;
}


// インターフェースリファレンス取得メソッド
// -----------------------------------------------------------
- (int)_findInterface
{
	SInt32						score;
	IOReturn					error;
    IOCFPlugInInterface			**iodev;
    IOUSBFindInterfaceRequest	interfaceRequest;
    io_iterator_t				iterator;
	
	// プラグインインターフェースの生成
	error = IOCreatePlugInInterfaceForService(
		usbDeviceRef,
		kIOUSBDeviceUserClientTypeID,
		kIOCFPlugInInterfaceID,
		&iodev,
		&score
	);
	if(error) {
		printf("_findInterface: Unable to create a device interface.\n");
		return 0;
    }
	error = (*iodev)->QueryInterface(
		iodev, 
		CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID245),
		(LPVOID)&dev
	);
	IODestroyPlugInInterface(iodev);
	if(error || !dev) {
		printf("_findInterface: Unable to create a device interface.\n");
		return 0;
    }
	
	// デバイスのオープン
	error = (*dev)->USBDeviceOpen(dev);
	if(error) {
		printf("_findInterface: Unable to open device.\n");
		return 0;
    }
	
	// インターフェースイテレータの生成
	interfaceRequest.bInterfaceClass = kIOUSBFindInterfaceDontCare;
    interfaceRequest.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
    interfaceRequest.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
	interfaceRequest.bAlternateSetting = kIOUSBFindInterfaceDontCare;
	error = (*dev)->CreateInterfaceIterator(dev, &interfaceRequest, &iterator);
	if(error) {
		printf("_findInterface: Unable to create interface iterator.\n");
		(*dev)->USBDeviceClose(dev);
        (*dev)->Release(dev);
		return 0;
	}
	
	// インターフェースリファレンスの取得
	usbInterfaceRef = IOIteratorNext(iterator);
	
	// 後処理
	IOObjectRelease(iterator);
    iterator = 0;
	
	return 1;
}


// インターフェースセットアップメソッド
// -----------------------------------------------------------
- (int)_setupInterface
{
    SInt32					score;
	IOReturn				error;
    IOCFPlugInInterface 	**iodev;

	// プラグインインターフェースの生成
	error = IOCreatePlugInInterfaceForService(
		usbInterfaceRef,
		kIOUSBInterfaceUserClientTypeID,
		kIOCFPlugInInterfaceID,
		&iodev,
		&score
	);
	error = (*iodev)->QueryInterface(
		iodev,
		CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID245),
		(LPVOID)&interface
	);
	IODestroyPlugInInterface(iodev);
	if(error || !interface){
		printf("_setupInterface: Unable to create a device interface.\n");
		return 0;
    }
	
	// インターフェースを開く
	error = (*interface)->USBInterfaceOpen(interface);
	if(error) {
		printf("_setupInterface: Unable to open interface.\n");
		return 0;
    }
	
	return 1;
}


// クローズメソッド
// -----------------------------------------------------------
- (int)close
{
    IOReturn error;
	
	// インターフェースを閉じる
	error = (*interface)->USBInterfaceClose(interface);
    if(error) {
		printf("close: Unable to close interface.\n");
		return 0;
    }
	// インターフェースを解放
	error = (*interface)->Release(interface);
	if(error) {
		printf("close: Unable to release interface.\n");
		return 0;
    }

	// デバイスを閉じる
	error = (*dev)->USBDeviceClose(dev);
    if(error) {
		printf("close: Unable to close device.\n");
		return 0;
	}
	// デバイスの解放
	error = (*dev)->Release(dev);
	if(error) {
		printf("close: Unable to release device.\n");
		return 0;
    }
	
	// マスターポートの解放
    mach_port_deallocate(mach_task_self(), masterPort);
	return 1;
}


// リクエスト送信関数
// -----------------------------------------------------------
- (int)_sendRequest:(UInt16)wValue withIndex:(UInt16)wIndex
{
	unsigned char	iobuff[32];
	IOReturn		error;
	
	// リクエストを生成
	IOUSBDevRequest req;
	req.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req.bRequest = 0x01;
	req.wValue = wValue;
	req.wIndex = wIndex;
	req.wLength = 0;
	req.wLenDone = 0;
	req.pData = iobuff;
	
	// Friioに送信
	error = (*interface)->ControlRequest(interface, 0, &req);
	if(error) {
		return 0;
	}
	
	return 1;
}	


// 初期化処理Aメソッド
// -----------------------------------------------------------
- (int)initStageA
{
	int			error_flag;
	IOReturn	error;

	// リクエスト送信
	error_flag = [self _sendRequest:0x0002 withIndex:0x0011];
	// リクエストの送信に失敗したら初期化失敗
	if(!error_flag) {
		return 0;
	}
	[self _sendRequest:0x0000 withIndex:0x0011];
	
	unsigned char iobuff1[32];
	IOUSBDevRequest req1;
	req1.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req1.bRequest = 0x02;
	req1.wValue = 0xA000;
	req1.wIndex = 0x00FE;
	req1.wLength = 1;
	req1.wLenDone = 0;
	req1.pData = iobuff1;
	error = (*interface)->ControlRequest(interface, 0, &req1);

	unsigned char iobuff2[32];
	IOUSBDevRequest req2;
	req2.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req2.bRequest = 0x02;
	req2.wValue = 0x3000;
	req2.wIndex = 0x0080;
	req2.wLength = 1;
	req2.wLenDone = 0;
	req2.pData = iobuff2;
	error = (*interface)->ControlRequest(interface, 0, &req2);
	
	[self _sendRequest:0x0004 withIndex:0x0030];
	
	return 1;
}


// 初期化処理Aメソッド
// -----------------------------------------------------------
- (int)initStageB
{
	[self _sendRequest:0x3040 withIndex:0x01];
	[self _sendRequest:0x3038 withIndex:0x04];
	[self _sendRequest:0x3040 withIndex:0x05];
	[self _sendRequest:0x3040 withIndex:0x07];
	[self _sendRequest:0x304F withIndex:0x0F];
	[self _sendRequest:0x3021 withIndex:0x11];
	[self _sendRequest:0x300B withIndex:0x12];
	[self _sendRequest:0x302F withIndex:0x13];
	[self _sendRequest:0x3031 withIndex:0x14];
	[self _sendRequest:0x3002 withIndex:0x16];
	[self _sendRequest:0x30C4 withIndex:0x21];
	[self _sendRequest:0x3020 withIndex:0x22];
	[self _sendRequest:0x3079 withIndex:0x2C];
	[self _sendRequest:0x3034 withIndex:0x2D];
	[self _sendRequest:0x3000 withIndex:0x2F];
	[self _sendRequest:0x3028 withIndex:0x30];
	[self _sendRequest:0x3031 withIndex:0x31];
	[self _sendRequest:0x30DF withIndex:0x32];
	[self _sendRequest:0x3001 withIndex:0x38];
	[self _sendRequest:0x3078 withIndex:0x39];
	[self _sendRequest:0x3033 withIndex:0x3B];
	[self _sendRequest:0x3033 withIndex:0x3C];
	[self _sendRequest:0x3090 withIndex:0x48];
	[self _sendRequest:0x3068 withIndex:0x51];
	[self _sendRequest:0x3038 withIndex:0x5E];
	[self _sendRequest:0x3000 withIndex:0x71];
	[self _sendRequest:0x3008 withIndex:0x72];
	[self _sendRequest:0x3000 withIndex:0x77];
	[self _sendRequest:0x3021 withIndex:0xC0];
	[self _sendRequest:0x3010 withIndex:0xC1];
	[self _sendRequest:0x301A withIndex:0xE4];
	[self _sendRequest:0x301F withIndex:0xEA];
	[self _sendRequest:0x3000 withIndex:0x77];
	[self _sendRequest:0x3000 withIndex:0x71];
	[self _sendRequest:0x3000 withIndex:0x71];
	[self _sendRequest:0x300C withIndex:0x76];
	
	return 1;
}


// ストリーム初期化メソッド
// -----------------------------------------------------------
- (int)initStream
{
	[self _sendRequest:0x08 withIndex:0x33];
	[self _sendRequest:0x40 withIndex:0x37];
	[self _sendRequest:0x1F withIndex:0x3A];
	[self _sendRequest:0xFF withIndex:0x3B];
	[self _sendRequest:0x1F withIndex:0x3C];
	[self _sendRequest:0xFF withIndex:0x3D];
	[self _sendRequest:0x00 withIndex:0x38];
	[self _sendRequest:0x00 withIndex:0x35];
	[self _sendRequest:0x00 withIndex:0x39];
	[self _sendRequest:0x00 withIndex:0x36];
	
	return 1;
}


// 録画実行メソッド
// -----------------------------------------------------------
- (int)rec:(char*)file withTime:(int)rec_sec
{
	// タイマーフラグとスレッドフラグを初期化
	TIMER_FLAG = 1;
	THREAD_FLAG = 1;
	
	// ファイルパスを保存
	file_path = file;
	
	// スレッドを生成
	NSThread *worker = [[NSThread alloc] initWithTarget:self selector:@selector(_recWorker:) object:nil];
	// スレッド起動
	[worker start];
	
	// 指定された時間待機
	time_t time_start = time(NULL);
	while(time(NULL) <= time_start + rec_sec) {
		printf(".");
		fflush(stdout);
		sleep(1);
	}
	// タイマーフラグを無効化
	TIMER_FLAG = 0;
	
	// スレッドを待つ
	while(THREAD_FLAG) {
		sleep(1);
	}
	// スレッド解放
	[worker release];
	
	return 0;
}


// 録画読み込みスレッド用メソッド
// -----------------------------------------------------------
- (void)_recWorker:(id)obj
{
	int					i;
	FILE				*output_fd;
	IOReturn			error;
    CFRunLoopSourceRef	cfSource;

	// 非同期イベントソースの生成
	error = (*interface)->CreateInterfaceAsyncEventSource(interface, &cfSource);
	if(error) {
		printf("_recWorker: Unable to create event source.\n");
		return;
    }
	// イベントソースを登録
	CFRunLoopAddSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
	
	// 出力先のファイルデスクリプタを取得
	output_fd = fopen(file_path, "w");
	if(output_fd == NULL) {
		printf("_recWorker: Unable to open output file.\n");
		CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
		THREAD_FLAG = 0;
		return;
	}
	
	// バッファの初期化
	for(i=0; i<RESPONSE_SIZE; i++) {
		// インターフェースを初期化
		res[i].interface = interface;
		// バッファを初期化
		memset(res[i].buffer, '\0', sizeof(res[i].buffer));
		// ファイルデスクリプタを保存
		res[i].fd = output_fd;
	}
	
	// おまじない
	(*interface)->SetAlternateInterface(interface, 1);
	(*interface)->SetAlternateInterface(interface, 0);

	// パイプ初期化
	error = (*interface)->ClearPipeStall(interface, (UInt8)1);
	if(error) {
		printf("_recWorker: Could not clear the pipe.\n");
	}
	
	// 非同期Bulk転送 1本目
	error = (*interface)->ReadPipeAsync(interface, 1, res[RESPONSE_NUM].buffer, BUFFER_SIZE, (IOAsyncCallback1)_workerCallback, &res[RESPONSE_NUM]);
	if(error) {
		printf("_recWorker: Read pipe error.\n");
		// 転送失敗時の後始末
		CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
		fflush(output_fd);
		fclose(output_fd);
		THREAD_FLAG = 0;
		return;
	}
	
	// 非同期Bulk転送 2本目
	RESPONSE_NUM++;
	error = (*interface)->ReadPipeAsync(interface, 1, res[RESPONSE_NUM].buffer, BUFFER_SIZE, (IOAsyncCallback1)_workerCallback, &res[RESPONSE_NUM]);
	if(error) {
		printf("_recWorker: Read pipe error.\n");
		// 転送失敗時の後始末
		CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
		fflush(output_fd);
		fclose(output_fd);
		THREAD_FLAG = 0;
		return;
	}
	
	// イベントループを実行
	CFRunLoopRun();

	// イベントループの後処理
	CFRunLoopRemoveSource(CFRunLoopGetCurrent(), cfSource, kCFRunLoopDefaultMode);
	fflush(output_fd);
	fclose(output_fd);
	THREAD_FLAG = 0;

}


// 録画読み込みスレッド用コールバック
// -----------------------------------------------------------
void _workerCallback(USBResponse *readRes, IOReturn result, void *arg) {
	IOReturn error;
	
	// コールバック結果のチェック
	if(result != kIOReturnSuccess) {
		printf("_workerCallback: Callback failed(%x).\n", result);
		CFRunLoopStop(CFRunLoopGetCurrent());
		return;
	}
	
	// パイプをクリア
	/*error = (*readRes->interface)->ClearPipeStall(readRes->interface, (UInt8)1);
	if(error) {
		printf("_workerCallback: Could not clear the pipe.\n");
	}*/
	
	// レスポンス番号を循環
	if(RESPONSE_NUM < RESPONSE_SIZE-1) {
		RESPONSE_NUM++;
	} else {
		RESPONSE_NUM = 0;
	}
	
	// 非同期Bulk転送を再実行
	error = (*res[RESPONSE_NUM].interface)->ReadPipeAsync(
		res[RESPONSE_NUM].interface,
		1,
		res[RESPONSE_NUM].buffer,
		BUFFER_SIZE,
		(IOAsyncCallback1)_workerCallback,
		&res[RESPONSE_NUM]
	);
	if(error) {
		printf("_workerCallback: Read pipe error.\n");
	}
	
	// ファイルに書き込み
	fwrite(readRes->buffer, 1, BUFFER_SIZE, readRes->fd);
	// 使ったバッファの初期化
	memset(readRes->buffer, '\0', sizeof(readRes->buffer));
	
	// 終了処理
	if(!TIMER_FLAG) {
		CFRunLoopStop(CFRunLoopGetCurrent());
	}
}


// チャンネル制御メソッド
// -----------------------------------------------------------
- (int)setChannel:(int)channel
{
	IOReturn error;

	// チャンネルのチェック
	if(channel < 13 || channel > 62) {
		printf("setChannel: Invalid channel.\n");
		return 0;
	}
	
	// チャンネルデータ計算
	const uint32_t chCode = 0x0E7F + 0x002A * (uint32_t)(channel - 13);
	
	// リクエスト用バッファ
	uint8_t sbuf[2][5];
	memset(sbuf, '\0', sizeof(sbuf));
	uint8_t rbuf[2][1];
	memset(rbuf, '\0', sizeof(rbuf));
	
	// 1回目リクエスト設定	
	sbuf[0][0] = 0xC0;
	sbuf[0][1] = (uint8_t)(chCode >> 8);
	sbuf[0][2] = (uint8_t)(chCode & 0xFF);
	sbuf[0][3] = 0xB2;
	sbuf[0][4] = 0x08;
	
	// 2回目リクエスト設定
	sbuf[1][0] = 0xC0;
	sbuf[1][1] = (uint8_t)(chCode >> 8);
	sbuf[1][2] = (uint8_t)(chCode & 0xFF);
	sbuf[1][3] = 0x9A;
	sbuf[1][4] = 0x50;
	
	// 1回目のリクエスト送信
	IOUSBDevRequest req1;
	req1.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req1.bRequest = 0x03;
	req1.wValue = 0x3000;
	req1.wIndex = 0x00FE;
	req1.wLength = sizeof(sbuf[0]);
	req1.wLenDone = 0;
	req1.pData = sbuf[0];
	error = (*interface)->ControlRequest(interface, 0, &req1);
	if(error) {
		printf("setChannel: Unable to set the channel.\n");
		return 0;
	}
	
	// 2回目のリクエスト送信
	IOUSBDevRequest req2;
	req2.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req2.bRequest = 0x03;
	req2.wValue = 0x3000;
	req2.wIndex = 0x00FE;
	req2.wLength = sizeof(sbuf[1]);
	req2.wLenDone = 0;
	req2.pData = sbuf[1];
	error = (*interface)->ControlRequest(interface, 0, &req2);
	if(error) {
		printf("setChannel: Unable to set the channel.\n");
		return 0;
	}	
	
	// 3回目のリクエスト送信
	IOUSBDevRequest req3;
	req3.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req3.bRequest = 0x02;
	req3.wValue = 0x3000;
	req3.wIndex = 0x00B0;
	req3.wLength = sizeof(rbuf[0]);
	req3.wLenDone = 0;
	req3.pData = rbuf[0];
	error = (*interface)->ControlRequest(interface, 0, &req3);
	if(error) {
		printf("setChannel: Unable to set the channel.\n");
		return 0;
	}
	
	// 4回目のリクエスト送信
	IOUSBDevRequest req4;
	req4.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req4.bRequest = 0x02;
	req4.wValue = 0x3000;
	req4.wIndex = 0x0080;
	req4.wLength = sizeof(rbuf[1]);
	req4.wLenDone = 0;
	req4.pData = rbuf[1];
	error = (*interface)->ControlRequest(interface, 0, &req4);
	if(error) {
		printf("setChannel: Unable to set the channel.\n");
		return 0;
	}
	
	return 1;
}



// シグナルレベル取得メソッド
// -----------------------------------------------------------
- (float)getSignalLevel
{
	IOReturn err;
	float signal_level = 0;
	uint8_t sigbuf[37];
	uint32_t dwHex;
	
	// リクエストバッファ
	memset(sigbuf, '\0', sizeof(sigbuf));

	// リクエスト送信
	IOUSBDevRequest req;
	req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req.bRequest = 0x02;
	req.wValue = 0x3000;
	req.wIndex = 0x0089;
	req.wLength = sizeof(sigbuf);
	req.wLenDone = 0;
	req.pData = sigbuf;
	err = (*interface)->ControlRequest(interface, 0, &req);
	if(err) {
		printf("setChannel: Unable to get the signal level.\n");
		return 0;
	}

	// シグナルレベルを算出
	dwHex = ((uint32_t)(sigbuf[2]) << 16) | ((uint32_t)(sigbuf[3]) << 8) | (uint32_t)(sigbuf[4]);
	if(!dwHex) {
		signal_level = 0.0f;
	} else {
		signal_level = (float)(68.4344993f - log10((double)dwHex) * 10.0 + pow(log10((double)dwHex) - 5.045f, (double)4.0f) / 4.09f);
	}
	
	return signal_level;	
}


// LED制御メソッド
// -----------------------------------------------------------
- (void)setLED:(int)color
{
	[self _sendRequest:0x000F withIndex:0x06];
	switch(color) {
		// 紫
		case 0:
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0009 withIndex:0x00];
			[self _sendRequest:0x000D withIndex:0x00];
			break;
		// 緑
		case 1:
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0001 withIndex:0x00];
			[self _sendRequest:0x0005 withIndex:0x00];
			break;
		// 赤
		case 2:
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x000B withIndex:0x00];
			[self _sendRequest:0x000F withIndex:0x00];
			[self _sendRequest:0x0003 withIndex:0x00];
			[self _sendRequest:0x0007 withIndex:0x00];
			[self _sendRequest:0x0001 withIndex:0x00];
			[self _sendRequest:0x0005 withIndex:0x00];
			break;
		// 白
		case 3:
			// 情報なし
			break;
	}
	[self _sendRequest:0x0000 withIndex:0x06];
}


@end
