#import <Foundation/Foundation.h>
#import "Friio.h"

int main (int argc, const char * argv[]) {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	const int PATH_SIZE = 512;
	int	i, error_flag = 1, b25_flag = 0;
	float signal = 0;
	
	// コマンド引数のチェック
	if(argc < 4) {
		printf("Usage: %s recsec channel outputfile (-b25)\n", argv[0]);
		exit(1);
	}

	// 引数を受け取り
	int recsec  = atoi(argv[1]);
	int channel = atoi(argv[2]);
	const char *output_path = argv[3];
	
	// 各種パス用配列
	char target_path[PATH_SIZE];
	char temp_path[PATH_SIZE];
	char b25_path[PATH_SIZE];
	
	// b25フラグのチェック
	if(argc==5 && (strcmp(argv[4], "-b25")==0)) {
		b25_flag = 1;
	}
	
	if(b25_flag) {
		// 実行ディレクトリのパスを取得
		char bin_dir[PATH_SIZE];
		strcpy(bin_dir, argv[0]);
		for(i=strlen(bin_dir)-1; i>=0; i--) {
			if(bin_dir[i] == '/') {
				bin_dir[i] = '\0';
				break;
			}
		}
	
		// 一時ファイルのパス
		const char *temp = ".tmp";
		sprintf(temp_path, "%s%s", output_path, temp);
		// b25のパス
		sprintf(b25_path, "%s/b25", bin_dir);
		
		// ターゲットパス
		strcpy(target_path, temp_path);
	} else {
		// ターゲットパス
		strcpy(target_path, output_path);	
	}
	
	// 引数の値チェック
	if(channel < 13 || 62 < channel) {
		printf("Channel number was wrong.\n");
		exit(1);	
	}
	if(recsec <= 0) {
		printf("Recsec was wrong.\n");
		exit(1);	
	}
	
	// Friioインスタンスを生成
	Friio* friio = [Friio alloc];
	
	// Friioを開く
	error_flag = [friio open];
	if(!error_flag) {
		printf("Unable to open the friio.\n");
		exit(1);
	}
	
	// 初期化A
	error_flag = [friio initStageA];
	// 初期化Aで失敗したらエラー
	if(!error_flag) {
		printf("Initialization failed.\n");
		exit(1);
	}
	// ストリームの初期化
	[friio initStream];
	// 初期化B
	[friio initStageB];
	// 初期化完了
	printf("Initialization complete.\n");

	// LEDを変更
	[friio setLED:1]; // 緑
	
	// チャンネルを変更
	error_flag = [friio setChannel:channel];
	// チャンネル変更で失敗したらエラー
	if(!error_flag) {
		[friio setLED:0];
		exit(1);
	}
	printf("Set channel: %d\n", channel);
	sleep(1);
	
	for(i=1; i<=3; i++) {
		// シグナルを取得
		signal = [friio getSignalLevel];
		printf("Signal level: %f\n", signal);
		// シグナル値のチェック
		if(signal > 15) {
			break;
		}
		sleep(1);
	}
	if(signal < 15) {
		printf("Signal level is insufficient.\n");
		[friio setLED:0];
		exit(1);
	}

	// LEDを変更
	[friio setLED:2]; // 赤

	// 録画実行
	printf("Recording");
	fflush(stdout);
	[friio rec:target_path withTime:recsec];
	printf("done.\n");
	
	// LEDを変更
	[friio setLED:0]; // 紫

	// Friioを閉じる
	error_flag = [friio close];
	if(!error_flag) {
		printf("Unable to close the friio.\n");
	}
	[friio release];
	
	if(b25_flag) {
		// 暗号解除
		printf("Decrypting start.\n");
		printf("=========================\n");
		char command[(PATH_SIZE*2)];
		sprintf(command, "%s %s %s", b25_path, temp_path, output_path);
		system(command);
		printf("=========================\n");
		printf("Decrypting complete.\n");
	
		// 一時ファイルを削除
		int error = unlink(temp_path);
		if(error) {
			printf("Could not remove the temporary file.\n");
		}
	}
	
    [pool drain];
	
    return 0;
}
